<?php

// Load core class
require_once 'PrizmPayClient.php';

/**
 * Prizm Pay - API Helper
 * 
 * The class for generate API request
 * 
 * @version 1.1
 */
class PrizmPayClient_API {

    /**
     * @var string API URL 
     */
    protected $_url;

    /**
     * @var int Merchant ID 
     */
    protected $_mid;

    /**
     * @var string Secret key 
     */
    protected $_key;

    /**
     * @var string The content in card vault key file
     */
    protected $_cardVaultKey;

    /**
     * @var string Last request data 
     */
    protected $_lastRequest = '';

    /**
     * @var string Last response
     */
    protected $_lastResponse = '';

    /**
     * @var string Last error message 
     */
    protected $_lastError = '';

    /**
     * 
     * @param string $url           API URL
     * @param int $merchantID       Merchant ID
     * @param string $secretKey     Merchant Secret Key
     */
    public function __construct($url, $merchantID, $secretKey) {
        $this->setURL($url);
        $this->setMerchant($merchantID, $secretKey);
    }

    /**
     * Set merchant information
     * 
     * @param int $merchantID       Merchant ID
     * @param string $secretKey     Merchant Sercet Key
     */
    public function setMerchant($merchantID, $secretKey) {
        $this->_mid = $merchantID;
        $this->_key = $secretKey;
    }

    /**
     * Set payment page url
     * 
     * @param string $url
     */
    public function setURL($url) {
        if (substr($url, -1) != '/') {
            $url .= '/';
        }
        $this->_url = $url;
    }

    public function ping($values = array()) {
        return $this->request('ping', $values);
    }

    /**
     * Sale API
     * 
     * @param array $data
     * @return mixed
     */
    public function sale($data) {
        $data = $this->processData($data);
        return $this->request('sale', array('data' => json_encode($data)));
    }

    /**
     * Authorize API
     * 
     * @param array $data
     * @return mixed
     */
    public function authorize($data) {
        $data = $this->processData($data);
        return $this->request('authorize', array('data' => json_encode($data)));
    }

    /**
     * Sale By Apple Pay API
     *
     * @param array $data
     * @return mixed
     */
    public function saleByApplePay($data) {
        $data = $this->processData($data);
        return $this->request('apple-pay/sale', array('data' => json_encode($data)));
    }

    /**
     * Authorize By Apple Pay API
     *
     * @param array $data
     * @return mixed
     */
    public function authorizeByApplePay($data) {
        $data = $this->processData($data);
        return $this->request('apple-pay/authorize', array('data' => json_encode($data)));
    }

    /**
     * Capture API
     * 
     * @param array $data
     * @return mixed
     */
    public function capture($data) {
        $data = $this->processData($data);
        return $this->request('capture', array('data' => json_encode($data)));
    }

    /**
     * Reverse API
     * 
     * @param array $data
     * @return mixed
     */
    public function reverse($data) {
        $data = $this->processData($data);
        return $this->request('reverse', array('data' => json_encode($data)));
    }

    /**
     * Refund API
     * 
     * @param array $data
     * @return mixed
     */
    public function refund($data) {
        return $this->request('refund', array('data' => json_encode($data)));
    }

    /**
     * 3DS enrollment checking API
     * Perform authorize after completed the payer authentication validation
     *
     * @param array $data
     *
     * @return mixed
     */
    public function check3DSEnrollmentForAuthorize($data) {
        $data = $this->processData($data);
        return $this->request('payer-auth/check-3ds-enroll-authorize', array('data' => json_encode($data)));
    }

    /**
     * 3DS enrollment checking API
     * Perform sale after completed the payer authentication validation
     *
     * @param array $data
     *
     * @return mixed
     */
    public function check3DSEnrollmentForSale($data) {
        $data = $this->processData($data);
        return $this->request('payer-auth/check-3ds-enroll-sale', array('data' => json_encode($data)));
    }

    /**
     * Decrypt Apple Pay Payment Token using Merchant Decryption
     *
     * @param string $paymentToken Payment Token in Json Format
     *
     * @return mixed
     */
    public function decryptApplePayTokenByMerchant($paymentToken) {
        $data                 = $this->processData(array());
        $data['paymentToken'] = $paymentToken;
        return $this->request('apple-pay/decrypt-token-by-merchant', array('data' => json_encode($data)));
    }

    /**
     * Remove card vault by token
     *
     * @param string $cardVaultToken Card Vault Token
     *
     * @return mixed
     */
    public function removeCardVault($cardVaultToken) {
        $data                    = $this->processData(array());
        $data['cardVault_token'] = $cardVaultToken;
        return $this->request('card-vault/remove', array('data' => json_encode($data)));
    }

    /**
     * process Data
     * 
     * @param array $data
     * @return array
     */
    public function processData($data) {
        $data['merchantID'] = $this->_mid;
        if (isset($data['merchant_referenceCode'])) {
            $data['merchantReferenceCode'] = $data['merchant_referenceCode'];
            unset($data['merchant_referenceCode']);
        }
        if (isset($data['merchant_invoiceId'])) {
            $data['merchantInvoiceID'] = $data['merchant_invoiceId'];
            unset($data['merchant_invoiceId']);
        }
        if (isset($data['purchaseTotals_grandTotalAmount'])) {
            $data['purchaseTotals']['grandTotalAmount'] = $data['purchaseTotals_grandTotalAmount'];
            unset($data['purchaseTotals_grandTotalAmount']);
        }
        if (isset($data['purchaseTotals_currency'])) {
            $data['purchaseTotals']['currency'] = $data['purchaseTotals_currency'];
            unset($data['purchaseTotals_currency']);
        }
        return $data;
    }

    /**
     * Request API
     * 
     * @param string $api
     * @param array $params
     * @return mixed
     */
    public function request($api, $params) {

        if (empty($this->_url)) {
            die('Error: please set the api url first');
        }

        $url = $this->_url . $api;

        $time    = time();
        $message = $this->_key . $time . $this->_mid . implode($params);
        $hash    = hash('sha256', $message);

        $headers = array(
            'API_VERSION: ' . 100,
            'API_MID: ' . $this->_mid,
            'API_TIME: ' . $time,
            'API_HASH: ' . $hash,
        );

        $postField = http_build_query($params);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_VERBOSE, TRUE);
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postField);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLINFO_HEADER_OUT, TRUE);

        $curlResult          = curl_exec($ch);
        $this->_lastResponse = $curlResult;
        $this->_lastRequest  = curl_getinfo($ch, CURLINFO_HEADER_OUT) . $postField;

        if ($curlResult === FALSE) {
            $this->_lastError = curl_error($ch);
        }

        curl_close($ch);
        if ($curlResult && ($curlResult[0] == '{')) {
            return json_decode($curlResult, TRUE);
        }
        return FALSE;
    }

    /**
     * Get/Print debug information
     * 
     * @param bool $print
     * @return string
     */
    public function debug($print = FALSE) {

        $o = '<pre>';
        $o .= '<b>Request:</b> ' . PHP_EOL . $this->_lastRequest . PHP_EOL . PHP_EOL;
        $o .= '<b>Response:</b> ' . PHP_EOL . $this->_lastResponse . PHP_EOL . PHP_EOL;

        if ($this->_lastResponse === FALSE) {
            $o .= PHP_EOL . '<b>Error:</b> ' . PHP_EOL . $this->_lastError . PHP_EOL;
        }
        $o .= '</pre>';
        if ($print) {
            echo $o;
        }
        return $o;
    }

}
