<?php

// Load core class
require_once 'PrizmPayClient.php';

/**
 * Prizm Pay - Standard Payment Page Helper
 * 
 * The class for
 * - format order data
 * - generate payment page request
 * 
 * @version 1.5
 */
class PrizmPayClient_PP {

    /**
     * @var array Order Information 
     */
    protected $_data;

    /**
     * @var int Merchant ID 
     */
    protected $_mid;

    /**
     * @var string Secret Key 
     */
    protected $_key;

    /**
     * @var string Target URL 
     */
    protected $_url;

    /**
     * @var array       All address fields 
     */
    protected $_addressFields = array(
        'firstName',
        'lastName',
        'email',
        'phoneNumber',
        'street1',
        'street2',
        'city',
        'state',
        'country',
        'postalCode');

    /**
     * @var array       All card fields
     */
    protected $_cardFields = array(
        'fullName',
        'accountNumber',
        'cvNumber',
		'cardType', 
        'expirationMonth',
        'expirationYear',
    );

    /**
     * 
     * @param string $url           Payment Page URL
     * @param int $merchantID       Merchant ID
     * @param string $secretKey     Merchant Secret Key
     */
    public function __construct($url = NULL, $merchantID = NULL, $secretKey = NULL) {
        $this->setURL($url);
        $this->setMerchant($merchantID, $secretKey);
    }

    /**
     * Set merchant information
     * 
     * @param int $mid      Merchant ID
     * @param string $key   Merchant Sercet Key
     */
    public function setMerchant($mid, $key) {
        $this->_mid = $mid;
        $this->_key = $key;
    }

    /**
     * Set payment page url
     * 
     * @param string $url
     */
    public function setURL($url) {
        $this->_url = $url;
    }

    /**
     * Set Field
     *
     * @param string $name
     * @param mixed  $value
     */
    public function setField($name, $value) {
        $this->_data[$name] = $value;
    }

    /**
     * Add order item
     * 
     * @param string $name
     * @param int $quantity
     * @param number $unitPrice
     */
    public function addItem($name, $quantity, $unitPrice) {
        $this->_data['item'][] = array(
            'name'      => (string) $name,
            'quantity'  => (string) $quantity,
            'unitPrice' => (string) $unitPrice,
        );
    }

    /**
     * Set order reference code
     * 
     * @param string $orderRef
     */
    public function setReferenceCode($orderRef) {
        $this->_data['merchant_referenceCode'] = (string) $orderRef;
    }

    /**
     * Set invoice no.
     * 
     * @param string $invoiceID
     */
    public function setInvoiceID($invoiceID) {
        $this->_data['merchant_invoiceId'] = (string) $invoiceID;
    }

    /**
     * Set invoice no.
     * 
     * @param string $ID
     */
    public function setDeviceFingerprintID($ID) {
        $this->_data['deviceFingerprintID'] = (string) $ID;
    }

    /**
     * Set grand total and currency
     * 
     * @param number $amount
     * @param string $currency
     */
    public function setGrandTotal($amount, $currency) {
        $this->_data['purchaseTotals_grandTotalAmount'] = (string) $amount;
        $this->_data['purchaseTotals_currency']         = strtoupper($currency);
    }

    /**
     * Setup card vault, store current request card to card vault
     * 
     * @param string $keyOrPath The key content or file path
     */
    public function setupCardVault($keyOrPath) {
        $this->_data['cardVault_setup'] = '1';
        $this->_setCardVaultKey($keyOrPath);
    }

    /**
     * Retrieve card data from card vault, for follow-on request only
     * 
     * @param string $keyOrPath The key content or file path
     * @param string $token The token string in first request
     */
    public function retrieveCardVault($keyOrPath, $token) {
        $this->_data['cardVault_token'] = $token;
        $this->_setCardVaultKey($keyOrPath);
    }

    /**
     * Set the card vault key
     * 
     * @param string $keyOrPath The key content or file path
     * @return boolean
     */
    protected function _setCardVaultKey($keyOrPath) {
        $ext = strtolower(substr($keyOrPath, -4));
        if ($ext === '.key') {
            if (!file_exists($keyOrPath) || !($key = file_get_contents($keyOrPath))) {
                return FALSE;
            }
        } else {
            $key = $keyOrPath;
        }

        $ps = explode('_', $key);
        if (count($ps) == 3) {
            $this->_data['cardVault_secretKey'] = trim($key);
            return TRUE;
        }

        return FALSE;
    }

    /**
     * Set address
     * 
     * @param string $section
     * @param string $firstName
     * @param string $lastName
     * @param string $email
     * @param string $phoneNumber
     * @param string $street1
     * @param string $street2
     * @param string $city
     * @param string $state
     * @param string $country
     * @param string $postalCode
     */
    protected function _setAddress($section, $firstName, $lastName, $email, $phoneNumber, $street1, $street2, $city, $state, $country, $postalCode) {

        $address = array();
        foreach ($this->_addressFields AS $k) {
            $address[$k] = $$k;
        }
        $this->_setAddressArray($section, $address);
    }

    /**
     * Set address with array
     * 
     * @param string $section
     * @param array $address
     */
    protected function _setAddressArray($section, $address) {

        foreach ($this->_addressFields AS $k) {
            $this->_data[$section][$k] = (string) isset($address[$k]) ? $address[$k] : '';
        }
    }

    /**
     * Set billing address
     * 
     * @param string $firstName
     * @param string $lastName
     * @param string $email
     * @param string $phoneNumber
     * @param string $street1
     * @param string $street2
     * @param string $city
     * @param string $state
     * @param string $country
     * @param string $postalCode
     */
    public function setBillingAddress($firstName, $lastName, $email, $phoneNumber, $street1, $street2, $city, $state, $country, $postalCode) {
        $this->_setAddress('billTo', $firstName, $lastName, $email, $phoneNumber, $street1, $street2, $city, $state, $country, $postalCode);
    }

    /**
     * Set billing address with data array
     * 
     * @param array $address
     */
    public function setBillingAddressArray($address) {
        $this->_setAddressArray('billTo', $address);
    }

    /**
     * Set shipping address
     * 
     * @param string $firstName
     * @param string $lastName
     * @param string $email
     * @param string $phoneNumber
     * @param string $street1
     * @param string $street2
     * @param string $city
     * @param string $state
     * @param string $country
     * @param string $postalCode
     */
    function setShippingAddress($firstName, $lastName, $email, $phoneNumber, $street1, $street2, $city, $state, $country, $postalCode) {
        $this->_setAddress('shipTo', $firstName, $lastName, $email, $phoneNumber, $street1, $street2, $city, $state, $country, $postalCode);
    }

    /**
     * Set shipping address with data array
     * 
     * @param array $address
     */
    function setShippingAddressArray($address) {
        $this->_setAddressArray('shipTo', $address);
    }

    /**
     * Set shipping address
     * 
     * @param string $firstName
     * @param string $lastName
     * @param string $email
     * @param string $phoneNumber
     * @param string $street1
     * @param string $street2
     * @param string $city
     * @param string $state
     * @param string $country
     * @param string $postalCode
     */
    function setCardHolderAddress($firstName, $lastName, $email, $phoneNumber, $street1, $street2, $city, $state, $country, $postalCode) {
        $this->_setAddress('cardHolder', $firstName, $lastName, $email, $phoneNumber, $street1, $street2, $city, $state, $country, $postalCode);
    }

    /**
     * Set shipping address with data array
     * 
     * @param array $address
     */
    function setCardHolderAddressArray($address) {
        $this->_setAddressArray('cardHolder', $address);
    }

    /**
     * Set card information
     * 
     * @param string $accountNumber
     * @param string $fullName
     * @param string $cvNumber
     * @param string $expirationMonth
     * @param string $expirationYear
     */
    function setCardInfo($accountNumber, $fullName, $cvNumber, $cardType, $expirationMonth, $expirationYear) {

        $card = array();
        foreach ($this->_cardFields AS $k) {
            $card[$k] = $$k;
        }
        $this->setCardInfoArray($card);
    }

    /**
     * Set card information with data array
     * 
     * @param array $card
     */
    function setCardInfoArray($card) {
        foreach ($card AS $key => $val) {
            $this->setCardInfoValue($key, $val);
        }
    }

    /**
     * Set specific card information value
     *
     * @param string $key
     * @param string $val
     */
    function setCardInfoValue($key, $val) {
        $this->_data['card'][$key] = (string)$val;
    }

    /**
     * Set custom variables
     *
     * @param array $variables
     */
    function setCustomVariables($variables) {
        $idx = 1;
        foreach ($variables as $_val) {
            $this->setField('customVariable_' . $idx++, $_val);
            if ($idx > 10) {
                break;
            }
        }
    }

    /**
     * Get data
     * 
     * @return array
     */
    function data() {
        return $this->_data;
    }

    /**
     * Open payment page
     */
    function open() {
        echo $this->html();
        exit;
    }

    /**
     * Generate html code
     * 
     * @return string HTML
     */
    function html($autoSubmit = true) {

        // json encode data
        $jsonData = json_encode($this->_data);

        // get current unix timestamp
        $unixTimestamp = time();

        // make secret hash
        $secretHash = PrizmPayClient::secretHash($this->_mid, $this->_key, $jsonData, $unixTimestamp);

        // make url
        $targetURL = "{$this->_url}/pay?mid={$this->_mid}&time={$unixTimestamp}&hash={$secretHash}";

        // create html form
        $html = "<!DOCTYPE html><html><head><meta charset='utf-8'></head><body><form id='form' name='form' method='post' action='{$targetURL}'>";

        // create hidden fields in form
        foreach ($this->_data AS $key => $val) {
            if (is_array($val)) {
                foreach ($val AS $k => $v) {
                    if (is_array($v)) {
                        foreach ($v AS $kk => $vv) {
                            $vv = htmlspecialchars($vv, ENT_QUOTES);
                            $html .= "<input type='hidden' name='data[{$key}][{$k}][{$kk}]' value='{$vv}' />";
                        }
                    } else {
                        $v = htmlspecialchars($v, ENT_QUOTES);
                        $html .= "<input type='hidden' name='data[{$key}][{$k}]' value='{$v}' />";
                    }
                }
            } else {
                $val = htmlspecialchars($val, ENT_QUOTES);
                $html .= "<input type='hidden' name='data[{$key}]' value='{$val}' />";
            }
        }
        $html .= "</form>";

        if ($autoSubmit) {
            // add javascript for auto submit
            $html .= "<script>document.getElementById('form').submit();</script>";
        }

        $html .= '</body></html>';

        return $html;
    }

}
